<?php

namespace Ems\portalGestor\Service;

use Zeedhi\Framework\Controller\Crud;
use Doctrine\ORM\EntityManager;

use Ems\portalGestor\Util\DateUtil;
use Ems\portalGestor\Util\Request;
use Ems\portalGestor\Util\Messages;
use Ems\portalGestor\Util\Util;
use Ems\portalGestor\Repository\ParamOrgRepository;
use Ems\portalGestor\Repository\OperadorRepository;
use Ems\portalGestor\Util\HcmServices;

use Ems\portalGestor\Util\ArrayUtil;

class OperadorService {

    /** @var ParamOrgRepository */
    protected $paramOrgRepository;

    /** @var OperadorRepository */
    protected $operadorRepository;
    /** @var HcmServices */
    protected $hcmServices;

    public const OPERATOR_DEFAULT_HASH = 'eea79e8ff3dd8b6e7160c5cc509c9bb6';
    public const DEFAULT_PASSWORD = 'HJ6bKz1mznUruX26R3Wp';

    public function __construct(
        EntityManager $entityManager,
        OperadorRepository $operadorRepository,
        ParamOrgRepository $paramOrgRepository,
        HcmServices $hcmServices
    ) {
        $this->entityManager        = $entityManager;
        $this->operadorRepository   = $operadorRepository;
        $this->paramOrgRepository   = $paramOrgRepository;
        $this->hcmServices          = $hcmServices;
    }

    public function getDadosOperador(
        string $cdoperador,
        string $dtNascPessoa        = null,
        string $dtAdmissaoVinc      = null,
        string $cpfPessoa           = null,
        string $idConfirmrecsenport = null
    ) {
        try {
            $dadosOperadores = $this->operadorRepository->getDadosOperador(
                $cdoperador,
                $dtNascPessoa,
                $dtAdmissaoVinc,
                $cpfPessoa,
                $idConfirmrecsenport
            );

            if (empty($dadosOperadores) || empty($dadosOperadores[0])) {
                throw new \Exception('Operador não encontrado');
            }

            $operador = $dadosOperadores[0];
            $nrOrg = $operador['NRORG'];

            $isDP = $this->operadorIsDP($nrOrg, $cdoperador);
            $operador['dp'] = $isDP;

            return $operador;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function validaLoginVinculo(string $nrvinculom, string $nrorg) {
        try {
            $loginValido = !empty($this->operadorRepository->validaLogin(
                $nrvinculom,
                $nrorg
            ));
            return $loginValido;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function validaLoginOperador(
        string $cdoperador,
        string $dtNascPessoa        = null,
        string $dtAdmissaoVinc      = null,
        string $cpfPessoa           = null,
        string $idConfirmrecsenport = null
    ) {
        $dadosOperador = $this->getDadosOperador(
            $cdoperador,
            $dtNascPessoa,
            $dtAdmissaoVinc,
            $cpfPessoa,
            $idConfirmrecsenport
        );

        $loginValido = !empty($dadosOperador);
        $dadosOperador['LOGIN_VALIDO'] = $loginValido;

        return $dadosOperador;
    }

    public function getPerfisDeAcesso(
        int $nrOrg,
        string $cdOperador
    ): array {
        $params = [
            'P_NRORG'      => $nrOrg,
            'P_CDOPERADOR' => $cdOperador
        ];

        $perfis = $this->operadorRepository->getPerfisDeAcesso($params);

        $nrPerfis = array_map(function($perfil) {
            return $perfil['NRPERFILACESSO'];
        }, $perfis);

        return $nrPerfis;
    }

    public function operadorIsDP(
        int $nrOrg,
        string $cdOperador
    ): bool {
        $nrPerfis = $this->getPerfisDeAcesso($nrOrg, $cdOperador);

        $paramOrg = $this->paramOrgRepository->getParamOrg($nrOrg);

        $stringPerfisDP = $paramOrg['NRPERFACESSADM'];
        $stringPerfisDP = is_null($stringPerfisDP) ? '' : $stringPerfisDP;

        // $perfisDP = ArrayUtil::fromStringSQL($stringPerfisDP, ',', false);
        $perfisDP = ArrayUtil::fromStringSQL($stringPerfisDP, ArrayUtil::COMMA_ARRAY_STRING_SEPARATOR, false);

        // $perfisDP = ArrayUtil::fromStringSQL($stringPerfisDP);
        $intersection = ArrayUtil::intersection($nrPerfis, $perfisDP);

        $isDP = count($intersection) > 0;

        return $isDP;
    }

    public function getVinculoCdOperador(
        int $nrOrg,
        int $nrVinculo
    ): string {
        $params = [
            'P_NRORG' => $nrOrg,
            'P_NRVINCULO' => $nrVinculo
        ];

        $cdOperador = $this->operadorRepository->getVinculoCdOperador($params);

        return $cdOperador;
    }

    public function vinculoIsDP(
        int $nrOrg,
        int $nrVinculo
    ): bool {
        $cdOperador = $this->getVinculoCdOperador($nrOrg, $nrVinculo);
        $isDP = $this->operadorIsDP($nrOrg, $cdOperador);

        return $isDP;
    }

    public function getBeforeForgotPasswordInfo(
        string  $email
    ): Array {
        $params = [
            'P_DSEMAILOPER' => $email,
        ];

        $dadosOperador = $this->operadorRepository->getBeforeForgotPasswordInfo($params);

        return $dadosOperador;
    }

    public function clearOperatorPassword(
        string  $nrOrg,
        string  $cdOperador
    ): bool {
        $params = [
            'P_NRORG'       => $nrOrg,
            'P_CDOPERADOR'  => $cdOperador,
        ];

        $dadosOperador = $this->operadorRepository->clearOperatorPassword($params);

        return $dadosOperador;
    }

    public function confirmationChangePassword(
        string  $nrOrg,
        string  $email,
        string  $cdOperador,
        string  $dtNascimento,
        string  $dtAdmissao,
        string  $cpf,
        string  $novaSenha,
        string  $novaSenhaConfirmacao
    ): bool {

        $cpfFinal = Util::sanitizeCpf($cpf);
        $defaultIdForConfirmation = 'S';

        // Validacao da confirmacao
        $isConfirmationValid = $this->validaLoginOperador(
            $cdOperador,
            $dtNascimento,
            $dtAdmissao,
            $cpfFinal,
            $defaultIdForConfirmation
        );

        if (empty($isConfirmationValid)) {
            throw new \Exception(Messages::LOGIN_USER_NOT_FOUND);
        }

        if ($novaSenha != $novaSenhaConfirmacao) {
            throw new \Exception(Messages::LOGIN_PASSW_NOT_MATCH);
        }

        // Redefine senha atual
        $this->clearOperatorPassword($nrOrg, $cdOperador);

        // Invoca o modulo de login para atualizaca da senha
        $response = Request::loginRequest(
            'updatePassword',
            [
                'json' => [
                    'requestType' => 'FilterData',
                    'filter' => [
                        [
                            'name' => 'ACTUAL_PASSWORD',
                            'operator' => '=',
                            'value' => self::DEFAULT_PASSWORD,
                        ],
                        [
                            'name' => 'NEW_PASSWORD',
                            'operator' => '=',
                            'value' => $novaSenha,
                        ],
                        [
                            'name' => 'CONFIRM_NEW_PASSWORD',
                            'operator' => '=',
                            'value' => $novaSenhaConfirmacao,
                        ],
                        [
                            'name' => 'EMAIL',
                            'operator' => '=',
                            'value' => $email,
                        ],
                    ]
                ]
            ]
        );
        
        $success = empty($response);
        if (!$success) {
            throw new \Exception(Messages::LOGIN_PASSW_UPDATE_FAIL . ' ' . $response['error']);
        }

        return $success;
    }

    public function restringeApenasMarcacaoPG(
        string $nrOrg,
        string $cdOperador
    ) : bool {

        $params = [
            'requestType' => 'Row',
            'row' => [
                'CDOPERADOR' => $cdOperador,
            ]
        ];

        $result = $this->hcmServices->send('restringeApenasMarcacaoPG', $params, $nrOrg);
        $success = $result['dataset']['restringeApenasMarcacaoPG']['success'];

        return $success;
    }

    public function removeRestricaoApenasMarcacaoPG(
        string $nrOrg,
        string $cdOperador
    ) : bool {

        $params = [
            'requestType' => 'Row',
            'row' => [
                'CDOPERADOR' => $cdOperador,
            ]
        ];

        $result = $this->hcmServices->send('removeRestricaoApenasMarcacaoPG', $params, $nrOrg);
        $success = $result['dataset']['removeRestricaoApenasMarcacaoPG']['success'];

        return $success;
    }
    
}